@@ -3,51 +3,51 @@
 ;;; Code:
 
 
-;;;### (autoloads nil "cal-bahai" "cal-bahai.el" (21710 25547 57569
-;;;;;;  864000))
+;;;### (autoloads nil "cal-bahai" "cal-bahai.el" (22180 39318 240078
+;;;;;;  271000))
 ;;; Generated autoloads from cal-bahai.el
 
 (autoload 'calendar-bahai-date-string "cal-bahai" "\
-String of Bahá'í date of Gregorian DATE.
+String of Bahá’í date of Gregorian DATE.
 Defaults to today's date if DATE is not given.
 
 \(fn &optional DATE)" nil nil)
 
 (autoload 'calendar-bahai-print-date "cal-bahai" "\
-Show the Bahá'í calendar equivalent of the selected date.
+Show the Bahá’í calendar equivalent of the selected date.
 
 \(fn)" t nil)
 
 (autoload 'calendar-bahai-goto-date "cal-bahai" "\
-Move cursor to Bahá'í date DATE; echo Bahá'í date unless NOECHO is non-nil.
+Move cursor to Bahá’í date DATE; echo Bahá’í date unless NOECHO is non-nil.
 
 \(fn DATE &optional NOECHO)" t nil)
 
 (autoload 'diary-bahai-insert-entry "cal-bahai" "\
 Insert a diary entry.
-For the Bahá'í date corresponding to the date indicated by point.
+For the Bahá’í date corresponding to the date indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
 (autoload 'diary-bahai-insert-monthly-entry "cal-bahai" "\
 Insert a monthly diary entry.
-For the day of the Bahá'í month corresponding to the date indicated by point.
+For the day of the Bahá’í month corresponding to the date indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
 (autoload 'diary-bahai-insert-yearly-entry "cal-bahai" "\
 Insert an annual diary entry.
-For the day of the Bahá'í year corresponding to the date indicated by point.
+For the day of the Bahá’í year corresponding to the date indicated by point.
 Prefix argument ARG will make the entry nonmarking.
 
 \(fn ARG)" t nil)
 
 ;;;***
 
-;;;### (autoloads nil "cal-china" "cal-china.el" (21710 25547 57569
-;;;;;;  864000))
+;;;### (autoloads nil "cal-china" "cal-china.el" (22180 39318 240078
+;;;;;;  271000))
 ;;; Generated autoloads from cal-china.el
 
 (autoload 'calendar-chinese-date-string "cal-china" "\
@@ -67,10 +67,57 @@ Echo Chinese date unless NOECHO is non-nil.
 
 \(fn DATE &optional NOECHO)" t nil)
 
+(autoload 'diary-chinese-mark-entries "cal-china" "\
+Mark days in the calendar window that have Chinese date diary entries.
+Marks each entry in `diary-file' (or included files) visible in the calendar
+window.  See `diary-chinese-list-entries' for more information.
+
+This function is provided for use with `diary-nongregorian-marking-hook'.
+
+\(fn)" nil nil)
+
+(autoload 'diary-chinese-list-entries "cal-china" "\
+Add any Chinese date entries from the diary file to `diary-entries-list'.
+Chinese date diary entries must be prefixed by `diary-chinese-entry-symbol'
+\(normally a `C').  The same `diary-date-forms' govern the style
+of the Chinese calendar entries.  If a Chinese date diary entry begins with
+`diary-nonmarking-symbol', the entry will appear in the diary listing,
+but will not be marked in the calendar.
+
+This function is provided for use with `diary-nongregorian-listing-hook'.
+
+\(fn)" nil nil)
+
+(autoload 'diary-chinese-anniversary "cal-china" "\
+Like `diary-anniversary' (which see) but accepts Chinese date.
+
+\(fn MONTH DAY &optional YEAR MARK)" nil nil)
+
+(autoload 'diary-chinese-insert-anniversary-entry "cal-china" "\
+Insert an anniversary diary entry for the Chinese date at point.
+Prefix argument ARG makes the entry nonmarking.
+
+\(fn &optional ARG)" t nil)
+
+(autoload 'diary-chinese-insert-entry "cal-china" "\
+Insert a diary entry for the Chinese date at point.
+
+\(fn &optional ARG)" t nil)
+
+(autoload 'diary-chinese-insert-monthly-entry "cal-china" "\
+Insert a monthly diary entry for the Chinese date at point.
+
+\(fn &optional ARG)" t nil)
+
+(autoload 'diary-chinese-insert-yearly-entry "cal-china" "\
+Insert a yearly diary entry for the Chinese date at point.
+
+\(fn &optional ARG)" t nil)
+
 ;;;***
 
-;;;### (autoloads nil "cal-coptic" "cal-coptic.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-coptic" "cal-coptic.el" (22180 39318 240078
+;;;;;;  271000))
 ;;; Generated autoloads from cal-coptic.el
 
 (autoload 'calendar-coptic-date-string "cal-coptic" "\
@@ -111,8 +158,8 @@ Echo Ethiopic date unless NOECHO is t.
 
 ;;;***
 
-;;;### (autoloads nil "cal-french" "cal-french.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-french" "cal-french.el" (22180 39318 240078
+;;;;;;  271000))
 ;;; Generated autoloads from cal-french.el
 
 (autoload 'calendar-french-date-string "cal-french" "\
@@ -135,8 +182,8 @@ Echo French Revolutionary date unless NOECHO is non-nil.
 
 ;;;***
 
-;;;### (autoloads nil "cal-hebrew" "cal-hebrew.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-hebrew" "cal-hebrew.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-hebrew.el
 
 (autoload 'calendar-hebrew-date-string "cal-hebrew" "\
@@ -178,8 +225,8 @@ Prefix argument ARG makes the entry nonmarking.
 
 ;;;***
 
-;;;### (autoloads nil "cal-html" "cal-html.el" (21710 25465 91935
-;;;;;;  286000))
+;;;### (autoloads nil "cal-html" "cal-html.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-html.el
 
 (autoload 'cal-html-cursor-month "cal-html" "\
@@ -201,8 +248,8 @@ specified by EVENT.  Note that any existing output files are overwritten.
 
 ;;;***
 
-;;;### (autoloads nil "cal-islam" "cal-islam.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-islam" "cal-islam.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-islam.el
 
 (autoload 'calendar-islamic-date-string "cal-islam" "\
@@ -246,7 +293,8 @@ Prefix argument ARG makes the entry nonmarking.
 
 ;;;***
 
-;;;### (autoloads nil "cal-iso" "cal-iso.el" (21710 25547 58569 859000))
+;;;### (autoloads nil "cal-iso" "cal-iso.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-iso.el
 
 (autoload 'calendar-iso-from-absolute "cal-iso" "\
@@ -284,8 +332,8 @@ Interactively, goes to the first day of the specified week.
 
 ;;;***
 
-;;;### (autoloads nil "cal-julian" "cal-julian.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-julian" "cal-julian.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-julian.el
 
 (autoload 'calendar-julian-from-absolute "cal-julian" "\
@@ -341,8 +389,8 @@ Echo astronomical (Julian) day number unless NOECHO is non-nil.
 
 ;;;***
 
-;;;### (autoloads nil "cal-mayan" "cal-mayan.el" (21710 25547 59569
-;;;;;;  855000))
+;;;### (autoloads nil "cal-mayan" "cal-mayan.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-mayan.el
 
 (autoload 'calendar-mayan-date-string "cal-mayan" "\
@@ -399,8 +447,8 @@ Echo Mayan date unless NOECHO is non-nil.
 
 ;;;***
 
-;;;### (autoloads nil "cal-move" "cal-move.el" (21710 25547 59569
-;;;;;;  855000))
+;;;### (autoloads nil "cal-move" "cal-move.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-move.el
 
 (autoload 'calendar-cursor-to-nearest-date "cal-move" "\
@@ -548,8 +596,8 @@ Negative DAY counts backward from end of year.
 
 ;;;***
 
-;;;### (autoloads nil "cal-persia" "cal-persia.el" (21710 25547 59569
-;;;;;;  855000))
+;;;### (autoloads nil "cal-persia" "cal-persia.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-persia.el
 
 (autoload 'calendar-persian-date-string "cal-persia" "\
@@ -570,7 +618,8 @@ Echo Persian date unless NOECHO is non-nil.
 
 ;;;***
 
-;;;### (autoloads nil "cal-tex" "cal-tex.el" (21710 25465 92935 282000))
+;;;### (autoloads nil "cal-tex" "cal-tex.el" (22180 39318 242078
+;;;;;;  261000))
 ;;; Generated autoloads from cal-tex.el
 
 (autoload 'cal-tex-cursor-year "cal-tex" "\
@@ -719,7 +768,7 @@ a buffer position to use instead of point.
 
 ;;;***
 
-;;;### (autoloads nil "cal-x" "cal-x.el" (21710 25547 59569 855000))
+;;;### (autoloads nil "cal-x" "cal-x.el" (22180 39318 242078 261000))
 ;;; Generated autoloads from cal-x.el
 
 (autoload 'calendar-frame-setup "cal-x" "\
@@ -736,38 +785,15 @@ If PROMPT is non-nil, prompt for the month and year to use.
 
 \(fn CONFIG &optional PROMPT)" nil nil)
 
-(autoload 'calendar-one-frame-setup "cal-x" "\
-Display calendar and diary in a single dedicated frame.
-See `calendar-frame-setup' for more information.
-
-\(fn &optional PROMPT)" nil nil)
-
-(make-obsolete 'calendar-one-frame-setup 'calendar-frame-setup '"23.1")
-
-(autoload 'calendar-only-one-frame-setup "cal-x" "\
-Display calendar in a dedicated frame.
-See `calendar-frame-setup' for more information.
-
-\(fn &optional PROMPT)" nil nil)
-
-(make-obsolete 'calendar-only-one-frame-setup 'calendar-frame-setup '"23.1")
-
-(autoload 'calendar-two-frame-setup "cal-x" "\
-Display calendar and diary in separate, dedicated frames.
-See `calendar-frame-setup' for more information.
-
-\(fn &optional PROMPT)" nil nil)
-
-(make-obsolete 'calendar-two-frame-setup 'calendar-frame-setup '"23.1")
-
 ;;;***
 
-;;;### (autoloads nil "diary-lib" "diary-lib.el" (21710 25547 60569
-;;;;;;  851000))
+;;;### (autoloads nil "diary-lib" "diary-lib.el" (22180 39318 244078
+;;;;;;  252000))
 ;;; Generated autoloads from diary-lib.el
 
 (autoload 'diary-live-p "diary-lib" "\
 Return non-nil if the diary is being displayed.
+The actual return value is a diary buffer.
 
 \(fn)" nil nil)
 
@@ -794,8 +820,6 @@ DFILE specifies the file to use as the diary file.
 
 \(fn ARG DFILE)" t nil)
 
-(define-obsolete-function-alias 'view-other-diary-entries 'diary-view-other-diary-entries "23.1")
-
 (autoload 'diary-show-all-entries "diary-lib" "\
 Show all of the diary entries in the diary file.
 This function gets rid of the selective display of the diary file so that
@@ -821,8 +845,6 @@ marks.  This is intended to deal with deleted diary entries.
 
 \(fn &optional REDRAW)" t nil)
 
-(define-obsolete-function-alias 'mark-diary-entries 'diary-mark-entries "23.1")
-
 (autoload 'diary-make-entry "diary-lib" "\
 Insert a diary entry STRING which may be NONMARKING in FILE.
 If omitted, NONMARKING defaults to nil and FILE defaults to
@@ -830,64 +852,48 @@ If omitted, NONMARKING defaults to nil and FILE defaults to
 
 \(fn STRING &optional NONMARKING FILE)" nil nil)
 
-(define-obsolete-function-alias 'make-diary-entry 'diary-make-entry "23.1")
-
 (autoload 'diary-insert-entry "diary-lib" "\
 Insert a diary entry for the date indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG &optional EVENT)" t nil)
 
-(define-obsolete-function-alias 'insert-diary-entry 'diary-insert-entry "23.1")
-
 (autoload 'diary-insert-weekly-entry "diary-lib" "\
 Insert a weekly diary entry for the day of the week indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-weekly-diary-entry 'diary-insert-weekly-entry "23.1")
-
 (autoload 'diary-insert-monthly-entry "diary-lib" "\
 Insert a monthly diary entry for the day of the month indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-monthly-diary-entry 'diary-insert-monthly-entry "23.1")
-
 (autoload 'diary-insert-yearly-entry "diary-lib" "\
 Insert an annual diary entry for the day of the year indicated by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-yearly-diary-entry 'diary-insert-yearly-entry "23.1")
-
 (autoload 'diary-insert-anniversary-entry "diary-lib" "\
 Insert an anniversary diary entry for the date given by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-anniversary-diary-entry 'diary-insert-anniversary-entry "23.1")
-
 (autoload 'diary-insert-block-entry "diary-lib" "\
 Insert a block diary entry for the days between the point and marked date.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-block-diary-entry 'diary-insert-block-entry "23.1")
-
 (autoload 'diary-insert-cyclic-entry "diary-lib" "\
 Insert a cyclic diary entry starting at the date given by point.
 Prefix argument ARG makes the entry nonmarking.
 
 \(fn ARG)" t nil)
 
-(define-obsolete-function-alias 'insert-cyclic-diary-entry 'diary-insert-cyclic-entry "23.1")
-
 (autoload 'diary-font-lock-keywords "diary-lib" "\
 Return a value for the variable `diary-font-lock-keywords'.
 
@@ -895,8 +901,8 @@ Return a value for the variable `diary-font-lock-keywords'.
 
 ;;;***
 
-;;;### (autoloads nil "holidays" "holidays.el" (21710 25547 60569
-;;;;;;  851000))
+;;;### (autoloads nil "holidays" "holidays.el" (22180 39318 244078
+;;;;;;  252000))
 ;;; Generated autoloads from holidays.el
 
 (autoload 'calendar-list-holidays "holidays" "\
@@ -924,7 +930,7 @@ use instead of point.
 
 ;;;***
 
-;;;### (autoloads nil "lunar" "lunar.el" (21710 25547 61569 846000))
+;;;### (autoloads nil "lunar" "lunar.el" (22180 39318 245078 248000))
 ;;; Generated autoloads from lunar.el
 
 (autoload 'calendar-lunar-phases "lunar" "\
@@ -934,11 +940,9 @@ use instead of point.
 
 \(fn &optional EVENT)" t nil)
 
-(define-obsolete-function-alias 'calendar-phases-of-moon 'calendar-lunar-phases "23.1")
-
 ;;;***
 
-;;;### (autoloads nil "solar" "solar.el" (21710 25465 95935 269000))
+;;;### (autoloads nil "solar" "solar.el" (22180 39318 245078 248000))
 ;;; Generated autoloads from solar.el
 
 (autoload 'calendar-sunrise-sunset "solar" "\
@@ -956,7 +960,7 @@ Local time of sunrise and sunset for month under cursor or at EVENT.
 
 ;;;### (autoloads nil nil ("appt.el" "cal-dst.el" "cal-menu.el" "calendar.el"
 ;;;;;;  "icalendar.el" "parse-time.el" "time-date.el" "timeclock.el"
-;;;;;;  "todo-mode.el") (21756 24807 779007 881000))
+;;;;;;  "todo-mode.el") (22180 39318 247078 239000))
 
 ;;;***
 
