@@ -3,17 +3,17 @@
 ;;; Code:
 
 
-;;;### (autoloads nil "cal-bahai" "cal-bahai.el" (21710 25547 57569
-;;;;;;  864000))
+;;;### (autoloads nil "cal-bahai" "cal-bahai.el" (22180 39318 240078
+;;;;;;  271000))
 ;;; Generated autoloads from cal-bahai.el
 
 (autoload 'diary-bahai-list-entries "cal-bahai" "\
-Add any Bahá'í date entries from the diary file to `diary-entries-list'.
-Bahá'í date diary entries must be prefaced by `diary-bahai-entry-symbol'
+Add any Bahá’í date entries from the diary file to `diary-entries-list'.
+Bahá’í date diary entries must be prefaced by `diary-bahai-entry-symbol'
 \(normally a `B').  The same diary date forms govern the style of the
-Bahá'í calendar entries, except that the Bahá'í month names cannot be
-abbreviated.  The Bahá'í months are numbered from 1 to 19 with Bahá being
-1 and 19 being `Alá.  If a Bahá'í date diary entry begins with
+Bahá’í calendar entries, except that the Bahá’í month names cannot be
+abbreviated.  The Bahá’í months are numbered from 1 to 19 with Bahá being
+1 and 19 being `Alá.  If a Bahá’í date diary entry begins with
 `diary-nonmarking-symbol', the entry will appear in the diary listing, but
 will not be marked in the calendar.  This function is provided for use with
 `diary-nongregorian-listing-hook'.
@@ -21,28 +21,28 @@ will not be marked in the calendar.  This function is provided for use with
 \(fn)" nil nil)
 
 (autoload 'calendar-bahai-mark-date-pattern "cal-bahai" "\
-Mark dates in calendar window that conform to Bahá'í date MONTH/DAY/YEAR.
+Mark dates in calendar window that conform to Bahá’í date MONTH/DAY/YEAR.
 A value of 0 in any position is a wildcard.  Optional argument COLOR is
 passed to `calendar-mark-visible-date' as MARK.
 
 \(fn MONTH DAY YEAR &optional COLOR)" nil nil)
 
 (autoload 'diary-bahai-mark-entries "cal-bahai" "\
-Mark days in the calendar window that have Bahá'í date diary entries.
+Mark days in the calendar window that have Bahá’í date diary entries.
 Marks each entry in `diary-file' (or included files) visible in the calendar
 window.  See `diary-bahai-list-entries' for more information.
 
 \(fn)" nil nil)
 
 (autoload 'diary-bahai-date "cal-bahai" "\
-Bahá'í calendar equivalent of date diary entry.
+Bahá’í calendar equivalent of date diary entry.
 
 \(fn)" nil nil)
 
 ;;;***
 
-;;;### (autoloads nil "cal-china" "cal-china.el" (21710 25547 57569
-;;;;;;  864000))
+;;;### (autoloads nil "cal-china" "cal-china.el" (22180 39318 240078
+;;;;;;  271000))
 ;;; Generated autoloads from cal-china.el
 
 (autoload 'diary-chinese-date "cal-china" "\
@@ -52,8 +52,8 @@ Chinese calendar equivalent of date diary entry.
 
 ;;;***
 
-;;;### (autoloads nil "cal-coptic" "cal-coptic.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-coptic" "cal-coptic.el" (22180 39318 240078
+;;;;;;  271000))
 ;;; Generated autoloads from cal-coptic.el
 
 (autoload 'diary-coptic-date "cal-coptic" "\
@@ -68,8 +68,8 @@ Ethiopic calendar equivalent of date diary entry.
 
 ;;;***
 
-;;;### (autoloads nil "cal-french" "cal-french.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-french" "cal-french.el" (22180 39318 240078
+;;;;;;  271000))
 ;;; Generated autoloads from cal-french.el
 
 (autoload 'diary-french-date "cal-french" "\
@@ -79,8 +79,8 @@ French calendar equivalent of date diary entry.
 
 ;;;***
 
-;;;### (autoloads nil "cal-hebrew" "cal-hebrew.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-hebrew" "cal-hebrew.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-hebrew.el
 
 (autoload 'diary-hebrew-list-entries "cal-hebrew" "\
@@ -98,8 +98,6 @@ is provided for use with `diary-nongregorian-listing-hook'.
 
 \(fn)" nil nil)
 
-(define-obsolete-function-alias 'list-hebrew-diary-entries 'diary-hebrew-list-entries "23.1")
-
 (autoload 'calendar-hebrew-mark-date-pattern "cal-hebrew" "\
 Mark dates in calendar window that conform to Hebrew date MONTH/DAY/YEAR.
 A value of 0 in any position is a wildcard.  Optional argument COLOR is
@@ -107,8 +105,6 @@ passed to `calendar-mark-visible-date' as MARK.
 
 \(fn MONTH DAY YEAR &optional COLOR)" nil nil)
 
-(define-obsolete-function-alias 'mark-hebrew-calendar-date-pattern 'calendar-hebrew-mark-date-pattern "23.1")
-
 (autoload 'diary-hebrew-mark-entries "cal-hebrew" "\
 Mark days in the calendar window that have Hebrew date diary entries.
 Marks each entry in `diary-file' (or included files) visible in the calendar
@@ -116,14 +112,6 @@ window.  See `list-hebrew-diary-entries' for more information.
 
 \(fn)" nil nil)
 
-(define-obsolete-function-alias 'mark-hebrew-diary-entries 'diary-hebrew-mark-entries "23.1")
-
-(define-obsolete-function-alias 'insert-hebrew-diary-entry 'diary-hebrew-insert-entry "23.1")
-
-(define-obsolete-function-alias 'insert-monthly-hebrew-diary-entry 'diary-hebrew-insert-monthly-entry "23.1")
-
-(define-obsolete-function-alias 'insert-yearly-hebrew-diary-entry 'diary-hebrew-insert-yearly-entry "23.1")
-
 (autoload 'diary-hebrew-date "cal-hebrew" "\
 Hebrew calendar equivalent of date diary entry.
 
@@ -154,8 +142,6 @@ use when highlighting the day in the calendar.
 
 \(fn &optional MARK)" nil nil)
 
-(define-obsolete-function-alias 'diary-omer 'diary-hebrew-omer "23.1")
-
 (autoload 'diary-hebrew-yahrzeit "cal-hebrew" "\
 Yahrzeit diary entry--entry applies if date is Yahrzeit or the day before.
 Parameters are DEATH-MONTH, DEATH-DAY, DEATH-YEAR; the diary
@@ -175,8 +161,6 @@ use when highlighting the day in the calendar.
 
 \(fn DEATH-MONTH DEATH-DAY DEATH-YEAR &optional MARK AFTER-SUNSET)" nil nil)
 
-(define-obsolete-function-alias 'diary-yahrzeit 'diary-hebrew-yahrzeit "23.1")
-
 (autoload 'diary-hebrew-rosh-hodesh "cal-hebrew" "\
 Rosh Hodesh diary entry.
 Entry applies if date is Rosh Hodesh, the day before, or the Saturday before.
@@ -186,8 +170,6 @@ use when highlighting the day in the calendar.
 
 \(fn &optional MARK)" nil nil)
 
-(define-obsolete-function-alias 'diary-rosh-hodesh 'diary-hebrew-rosh-hodesh "23.1")
-
 (autoload 'diary-hebrew-parasha "cal-hebrew" "\
 Parasha diary entry--entry applies if date is a Saturday.
 An optional parameter MARK specifies a face or single-character string to
@@ -205,12 +187,10 @@ use when highlighting the day in the calendar.
 
 \(fn &optional MARK)" nil nil)
 
-(define-obsolete-function-alias 'diary-sabbath-candles 'diary-hebrew-sabbath-candles "23.1")
-
 ;;;***
 
-;;;### (autoloads nil "cal-islam" "cal-islam.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-islam" "cal-islam.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-islam.el
 
 (autoload 'diary-islamic-list-entries "cal-islam" "\
@@ -248,7 +228,8 @@ Islamic calendar equivalent of date diary entry.
 
 ;;;***
 
-;;;### (autoloads nil "cal-iso" "cal-iso.el" (21710 25547 58569 859000))
+;;;### (autoloads nil "cal-iso" "cal-iso.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-iso.el
 
 (autoload 'diary-iso-date "cal-iso" "\
@@ -258,8 +239,8 @@ ISO calendar equivalent of date diary entry.
 
 ;;;***
 
-;;;### (autoloads nil "cal-julian" "cal-julian.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-julian" "cal-julian.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-julian.el
 
 (autoload 'diary-julian-date "cal-julian" "\
@@ -274,8 +255,8 @@ Astronomical (Julian) day number diary entry.
 
 ;;;***
 
-;;;### (autoloads nil "cal-mayan" "cal-mayan.el" (21710 25547 59569
-;;;;;;  855000))
+;;;### (autoloads nil "cal-mayan" "cal-mayan.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-mayan.el
 
 (autoload 'diary-mayan-date "cal-mayan" "\
@@ -285,8 +266,8 @@ Show the Mayan long count, haab, and tzolkin dates as a diary entry.
 
 ;;;***
 
-;;;### (autoloads nil "cal-persia" "cal-persia.el" (21710 25547 59569
-;;;;;;  855000))
+;;;### (autoloads nil "cal-persia" "cal-persia.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-persia.el
 
 (autoload 'diary-persian-date "cal-persia" "\
@@ -296,8 +277,8 @@ Persian calendar equivalent of date diary entry.
 
 ;;;***
 
-;;;### (autoloads nil "holidays" "holidays.el" (21710 25547 60569
-;;;;;;  851000))
+;;;### (autoloads nil "holidays" "holidays.el" (22180 39318 244078
+;;;;;;  252000))
 ;;; Generated autoloads from holidays.el
 
 (autoload 'calendar-holiday-list "holidays" "\
@@ -316,7 +297,7 @@ strings describing those holidays that apply on DATE, or nil if none do.
 
 ;;;***
 
-;;;### (autoloads nil "lunar" "lunar.el" (21710 25547 61569 846000))
+;;;### (autoloads nil "lunar" "lunar.el" (22180 39318 245078 248000))
 ;;; Generated autoloads from lunar.el
 
 (autoload 'diary-lunar-phases "lunar" "\
@@ -326,11 +307,9 @@ use when highlighting the day in the calendar.
 
 \(fn &optional MARK)" nil nil)
 
-(define-obsolete-function-alias 'diary-phases-of-moon 'diary-lunar-phases "23.1")
-
 ;;;***
 
-;;;### (autoloads nil "solar" "solar.el" (21710 25465 95935 269000))
+;;;### (autoloads nil "solar" "solar.el" (22180 39318 245078 248000))
 ;;; Generated autoloads from solar.el
 
 (autoload 'diary-sunrise-sunset "solar" "\
@@ -344,7 +323,7 @@ Accurate to a few seconds.
 ;;;### (autoloads nil nil ("appt.el" "cal-dst.el" "cal-html.el" "cal-loaddefs.el"
 ;;;;;;  "cal-menu.el" "cal-move.el" "cal-tex.el" "cal-x.el" "calendar.el"
 ;;;;;;  "diary-lib.el" "icalendar.el" "parse-time.el" "time-date.el"
-;;;;;;  "timeclock.el" "todo-mode.el") (21756 24808 904888 859000))
+;;;;;;  "timeclock.el" "todo-mode.el") (22188 49102 897834 221000))
 
 ;;;***
 
