@@ -3,20 +3,20 @@
 ;;; Code:
 
 
-;;;### (autoloads nil "cal-bahai" "cal-bahai.el" (21710 25547 57569
-;;;;;;  864000))
+;;;### (autoloads nil "cal-bahai" "cal-bahai.el" (22180 39318 240078
+;;;;;;  271000))
 ;;; Generated autoloads from cal-bahai.el
 
 (autoload 'holiday-bahai "cal-bahai" "\
-Holiday on MONTH, DAY (Bahá'í) called STRING.
-If MONTH, DAY (Bahá'í) is visible in the current calendar window,
+Holiday on MONTH, DAY (Bahá’í) called STRING.
+If MONTH, DAY (Bahá’í) is visible in the current calendar window,
 returns the corresponding Gregorian date in the form of the
 list (((month day year) STRING)).  Otherwise, returns nil.
 
 \(fn MONTH DAY STRING)" nil nil)
 
 (autoload 'holiday-bahai-new-year "cal-bahai" "\
-Holiday entry for the Bahá'í New Year, if visible in the calendar window.
+Holiday entry for the Bahá’í New Year, if visible in the calendar window.
 
 \(fn)" nil nil)
 
@@ -29,8 +29,8 @@ Only considers the first, ninth, and twelfth days, unless ALL or
 
 ;;;***
 
-;;;### (autoloads nil "cal-china" "cal-china.el" (21710 25547 57569
-;;;;;;  864000))
+;;;### (autoloads nil "cal-china" "cal-china.el" (22180 39318 240078
+;;;;;;  271000))
 ;;; Generated autoloads from cal-china.el
 
 (autoload 'holiday-chinese-new-year "cal-china" "\
@@ -61,8 +61,8 @@ Returns nil if it is not visible in the current calendar window.
 
 ;;;***
 
-;;;### (autoloads nil "cal-hebrew" "cal-hebrew.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-hebrew" "cal-hebrew.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-hebrew.el
 
 (autoload 'holiday-hebrew "cal-hebrew" "\
@@ -80,8 +80,6 @@ or ALL is non-nil.
 
 \(fn &optional ALL)" nil nil)
 
-(define-obsolete-function-alias 'holiday-rosh-hashanah-etc 'holiday-hebrew-rosh-hashanah "23.1")
-
 (autoload 'holiday-hebrew-hanukkah "cal-hebrew" "\
 List of dates related to Hanukkah, as visible in calendar window.
 Shows only Hanukkah, unless `calendar-hebrew-all-holidays-flag' or ALL
@@ -89,8 +87,6 @@ is non-nil.
 
 \(fn &optional ALL)" nil nil)
 
-(define-obsolete-function-alias 'holiday-hanukkah 'holiday-hebrew-hanukkah "23.1")
-
 (autoload 'holiday-hebrew-passover "cal-hebrew" "\
 List of dates related to Passover, as visible in calendar window.
 Shows only the major holidays, unless `calendar-hebrew-all-holidays-flag'
@@ -98,15 +94,11 @@ or ALL is non-nil.
 
 \(fn &optional ALL)" nil nil)
 
-(define-obsolete-function-alias 'holiday-passover-etc 'holiday-hebrew-passover "23.1")
-
 (autoload 'holiday-hebrew-tisha-b-av "cal-hebrew" "\
 List of dates around Tisha B'Av, as visible in calendar window.
 
 \(fn)" nil nil)
 
-(define-obsolete-function-alias 'holiday-tisha-b-av-etc 'holiday-hebrew-tisha-b-av "23.1")
-
 (autoload 'holiday-hebrew-misc "cal-hebrew" "\
 Miscellaneous Hebrew holidays, if visible in calendar window.
 Includes: Tal Umatar, Tzom Teveth, Tu B'Shevat, Shabbat Shirah, and
@@ -116,8 +108,8 @@ Kiddush HaHamah.
 
 ;;;***
 
-;;;### (autoloads nil "cal-islam" "cal-islam.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-islam" "cal-islam.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-islam.el
 
 (autoload 'holiday-islamic "cal-islam" "\
@@ -135,8 +127,8 @@ Holiday entry for the Islamic New Year, if visible in the calendar window.
 
 ;;;***
 
-;;;### (autoloads nil "cal-julian" "cal-julian.el" (21710 25547 58569
-;;;;;;  859000))
+;;;### (autoloads nil "cal-julian" "cal-julian.el" (22180 39318 241078
+;;;;;;  266000))
 ;;; Generated autoloads from cal-julian.el
 
 (autoload 'holiday-julian "cal-julian" "\
@@ -149,7 +141,7 @@ nil if it is not visible in the current calendar window.
 
 ;;;***
 
-;;;### (autoloads nil "solar" "solar.el" (21710 25465 95935 269000))
+;;;### (autoloads nil "solar" "solar.el" (22180 39318 245078 248000))
 ;;; Generated autoloads from solar.el
 
 (autoload 'solar-equinoxes-solstices "solar" "\
@@ -165,7 +157,7 @@ Requires floating point.
 ;;;;;;  "cal-mayan.el" "cal-menu.el" "cal-move.el" "cal-persia.el"
 ;;;;;;  "cal-tex.el" "cal-x.el" "calendar.el" "diary-lib.el" "diary-loaddefs.el"
 ;;;;;;  "holidays.el" "icalendar.el" "lunar.el" "parse-time.el" "time-date.el"
-;;;;;;  "timeclock.el" "todo-mode.el") (21756 24810 37893 713000))
+;;;;;;  "timeclock.el" "todo-mode.el") (22188 49105 829820 775000))
 
 ;;;***
 
