@@ -1,5 +1,5 @@
 /* Definitions for interface to indent.c
-   Copyright (C) 1985-1986, 2001-2013 Free Software Foundation, Inc.
+   Copyright (C) 1985-1986, 2001-2016 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -16,6 +16,11 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
+#ifndef EMACS_INDENT_H
+#define EMACS_INDENT_H
+
+#include "lisp.h"
+
 struct position
   {
     ptrdiff_t bufpos;
@@ -26,14 +31,14 @@ struct position
     int contin;
   };
 
-struct position *compute_motion (ptrdiff_t from, EMACS_INT fromvpos,
-                                 EMACS_INT fromhpos, bool did_motion,
-                                 ptrdiff_t to, EMACS_INT tovpos,
-                                 EMACS_INT tohpos,
+struct position *compute_motion (ptrdiff_t from, ptrdiff_t frombyte,
+				 EMACS_INT fromvpos, EMACS_INT fromhpos,
+				 bool did_motion, ptrdiff_t to,
+				 EMACS_INT tovpos, EMACS_INT tohpos,
                                  EMACS_INT width, ptrdiff_t hscroll,
                                  int tab_offset, struct window *);
-struct position *vmotion (ptrdiff_t from, EMACS_INT vtarget,
-                          struct window *);
+struct position *vmotion (ptrdiff_t from, ptrdiff_t from_byte,
+			  EMACS_INT vtarget, struct window *);
 ptrdiff_t skip_invisible (ptrdiff_t pos, ptrdiff_t *next_boundary_p,
                           ptrdiff_t to, Lisp_Object window);
 
@@ -51,3 +56,5 @@ bool disptab_matches_widthtab (struct Lisp_Char_Table *disptab,
 /* Recompute BUF's width table, using the display table DISPTAB.  */
 void recompute_width_table (struct buffer *buf,
                             struct Lisp_Char_Table *disptab);
+
+#endif /* EMACS_INDENT_H */
